package com.ruoyi.zdsz.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ruoyi.zdsz.domain.ZEngineeringInfrastructure;
import com.ruoyi.zdsz.domain.bo.ZEngineeringInfrastructureBo;
import com.ruoyi.zdsz.domain.bo.ZEngineeringNodeBo;
import com.ruoyi.zdsz.domain.vo.ZEngineeringCivilVo;
import com.ruoyi.zdsz.domain.vo.ZEngineeringInfrastructureVo;
import com.ruoyi.zdsz.mapper.ZEngineeringInfrastructureMapper;
import com.ruoyi.zdsz.service.IZEngineeringInfrastructureService;
import com.ruoyi.zdsz.service.IZEngineeringNodeService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 基建工程Service业务层处理
 *
 * @author ruoyi
 * @date 2024-01-03
 */
@RequiredArgsConstructor
@Service
public class ZEngineeringInfrastructureServiceImpl implements IZEngineeringInfrastructureService {

    @Resource
    private final ZEngineeringInfrastructureMapper baseMapper;

    @Resource
    private final IZEngineeringNodeService izEngineeringNodeService;

    /**
     * 查询基建工程
     */
    @Override
    public ZEngineeringInfrastructureVo queryById(String id){
        ZEngineeringInfrastructureVo vo = baseMapper.selectVoById(id);
        ZEngineeringNodeBo zEngineeringNodeBo = new ZEngineeringNodeBo();
        zEngineeringNodeBo.setCreateTime(vo.getCreateTime());
        zEngineeringNodeBo.setCivliId(vo.getId());
        vo.setzEngineeringNodeBoList(izEngineeringNodeService.queryListDetails(zEngineeringNodeBo));
        return vo;
    }

    /**
     * 查询基建工程列表
     */
    @Override
    public TableDataInfo<ZEngineeringInfrastructureVo> queryPageList(ZEngineeringInfrastructureBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZEngineeringInfrastructure> lqw = buildQueryWrapper(bo);
        Page<ZEngineeringInfrastructureVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询基建工程列表
     */
    @Override
    public List<ZEngineeringInfrastructureVo> queryList(ZEngineeringInfrastructureBo bo) {
        LambdaQueryWrapper<ZEngineeringInfrastructure> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZEngineeringInfrastructure> buildQueryWrapper(ZEngineeringInfrastructureBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZEngineeringInfrastructure> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getEnginName()), ZEngineeringInfrastructure::getEnginName, bo.getEnginName());
        lqw.eq(StringUtils.isNotBlank(bo.getEnginAddre()), ZEngineeringInfrastructure::getEnginAddre, bo.getEnginAddre());
        lqw.eq(StringUtils.isNotBlank(bo.getConstructUnit()), ZEngineeringInfrastructure::getConstructUnit, bo.getConstructUnit());
        lqw.eq(bo.getStartTime() != null, ZEngineeringInfrastructure::getStartTime, bo.getStartTime());
        lqw.eq(StringUtils.isNotBlank(bo.getPlannedDuration()), ZEngineeringInfrastructure::getPlannedDuration, bo.getPlannedDuration());
        lqw.eq(bo.getCompletedTime() != null, ZEngineeringInfrastructure::getCompletedTime, bo.getCompletedTime());
        lqw.eq(StringUtils.isNotBlank(bo.getEnginContent()), ZEngineeringInfrastructure::getEnginContent, bo.getEnginContent());
        return lqw;
    }

    /**
     * 新增基建工程
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean insertByBo(ZEngineeringInfrastructureBo bo) {
        ZEngineeringInfrastructure add = BeanUtil.toBean(bo, ZEngineeringInfrastructure.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
            ZEngineeringNodeBo zEngineeringNodeBo = bo.getzEngineeringNodeBo();
            zEngineeringNodeBo.setCivliId(add.getId());
            zEngineeringNodeBo.setCreateTime(add.getCreateTime());
            izEngineeringNodeService.insert(zEngineeringNodeBo);
        }
        return flag;
    }

    /**
     * 修改基建工程
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateByBo(ZEngineeringInfrastructureBo bo) {
        ZEngineeringInfrastructure update = BeanUtil.toBean(bo, ZEngineeringInfrastructure.class);
        validEntityBeforeSave(update);
        boolean flag = baseMapper.updateById(update) > 0;
        if (flag) {
            ZEngineeringInfrastructureVo vo = baseMapper.selectVoById(update.getId());
            ZEngineeringNodeBo zEngineeringNodeBo = bo.getzEngineeringNodeBo();
            zEngineeringNodeBo.setCivliId(vo.getId());
            zEngineeringNodeBo.setCreateTime(vo.getCreateTime());
            izEngineeringNodeService.update(zEngineeringNodeBo);
        }
        return flag;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZEngineeringInfrastructure entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除基建工程
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteWithValidByIds(Collection<String> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        ids.forEach(item -> {
            ZEngineeringInfrastructureVo vo = baseMapper.selectVoById(item);
            if(vo==null)
            {return;}
            ZEngineeringNodeBo zEngineeringNodeBo = new ZEngineeringNodeBo();
            zEngineeringNodeBo.setCreateTime(vo.getCreateTime());
            zEngineeringNodeBo.setCivliId(vo.getId());
            List<ZEngineeringNodeBo> zEngineeringNodeBos = izEngineeringNodeService.queryListDetails(zEngineeringNodeBo);
            zEngineeringNodeBos.forEach(ite -> izEngineeringNodeService.delete(ite));
        }); return baseMapper.deleteBatchIds(ids) > 0;
    }
}
